# Loads NZ Census area unit 2013 data 
# processes to ipf form and matching coding to Green Grid survey

require(spatialec) # for parameters etc

myPackages <- c(
  "data.table", # data munching
  "ggplot2", # plot linear imputation model results
  "here" # easy path management - https://speakerdeck.com/jennybc/zen-and-the-art-of-workflow-maintenance?slide=49
)
spatialec::loadLibraries(myPackages) # this will try to install packages it can't load
spatialec::setup()

# Load data ----

# NZ Area Unit data
# Loaded as separate files
# 2013 NZ Census data from NZ Stats at area unit level. For simplicity we use one file per constraint:
# 
#  * n people
#  * n dependent children
#  * fuel source (all counted - may cause confusion as sum to > 100% of households)
#  * n rooms
# 
# NB: these files, when downloaded form the [NZStats data extractor](http://nzdotstat.stats.govt.nz/wbos/Index.aspx?DataSetCode=TABLECODE8165#) come with higher levels of aggregation in the tables. These have to be removed by extracting just area unit rows.
# 


# Load areas labels ----

# First load area labels as we use these to select the right data rows.

areasDT <- data.table::fread(p_Params$areasTable2013)

auListDT <- areasDT[, .(nMBs = .N), keyby = .(AU2013_code, AU2013_label, REGC2013_label)]
auListDT <- auListDT[, AU2013_code := as.character(AU2013_code)] # for easier matching
setkey(auListDT, AU2013_code)

# fuelSource ----
fuelf <- paste0(p_Params$dataPath, "raw/areaUnits/fuelSource/TABLECODE8100_Data_47b7b3fc-0e40-431f-b313-141de4fb0013.csv")
message("Loading: ", fuelf)
fuelDT <- data.table::fread(fuelf)
message("N rows loaded: ", dkUtils::tidyNum(nrow(fuelDT)))
fuelDT <- fuelDT[, AU2013_code := AREA]
setkey(fuelDT, AU2013_code)
fuelDT <- fuelDT[auListDT]
message("N unique area units (fuel data): ", uniqueN(fuelDT$AU2013_code))

# create categories
# "value.heatSourceWood", "value.heatSourceElectricity", "value.heatSourceGas", "value.heatSourceCoal", "value.heatSourceOther"

fuelDT <- fuelDT[, censusConstraint := "heatSourceOther"] # complex one - note this contains 'None' as well
fuelDT <- fuelDT[`Fuel type used to heat dwelling` == "Total dwellings, fuel type used to heat dwelling",
                 censusConstraint := "fuel_totalHouseholds"]
fuelDT <- fuelDT[`Fuel type used to heat dwelling` == "Total dwellings stated",
                 censusConstraint := "fuel_totalStatedHouseholds"]
fuelDT <- fuelDT[`Fuel type used to heat dwelling` == "Wood",
                 censusConstraint := "heatSourceWood"]
fuelDT <- fuelDT[`Fuel type used to heat dwelling` %like% "Solar" | # <- what is this?
                   `Fuel type used to heat dwelling` == "Electricity",
                 censusConstraint := "heatSourceElectricity"]
fuelDT <- fuelDT[`Fuel type used to heat dwelling`  %like% "gas",
                 censusConstraint := "heatSourceGas"]
fuelDT <- fuelDT[`Fuel type used to heat dwelling` == "Coal",
                 censusConstraint := "heatSourceCoal"]
table(fuelDT$`Fuel type used to heat dwelling`, fuelDT$censusConstraint)

# convert to wide so variables in columns (for IPF)
fuelDT <- fuelDT[, count := Value]

fuel2013WDT <- reshape(fuelDT[YEAR == 2013,
                              .(AU2013_code,AU2013_label,censusConstraint,count)], 
                          idvar = c("AU2013_code", "AU2013_label"),
                          timevar = "censusConstraint",
                          direction = "wide")

setnames(fuel2013WDT, c("count.fuel_totalHouseholds", "count.fuel_totalStatedHouseholds",
                        "count.heatSourceElectricity", "count.heatSourceGas", 
                        "count.heatSourceWood", "count.heatSourceCoal", "count.heatSourceOther"),
         c("fuel_totalHouseholds", "fuel_totalStatedHouseholds",
                        "heatSourceElectricity", "heatSourceGas", 
                        "heatSourceWood", "heatSourceCoal", "heatSourceOther"))

# nKids ----
kidsf <- paste0(p_Params$dataPath, "raw/areaUnits/nKids/TABLECODE8141_Data_e6f03066-7bbf-4ba0-94b0-1821d5a4665a.csv")
message("Loading: ", kidsf)
kidsDT <- data.table::fread(kidsf)
message("N rows loaded: ", dkUtils::tidyNum(nrow(kidsDT)))
kidsDT <- kidsDT[, AU2013_code := AREA]
setkey(kidsDT, AU2013_code)
kidsDT <- kidsDT[auListDT]

message("N unique area units (kids data): ", uniqueN(kidsDT$AU2013_code))

# > create categories
# "nKids_0", "nKids_1m"
kidsDT <- kidsDT[, censusConstraint := "nKids_1m"] # we selected rows with dependent children in the census extractor
kidsDT <- kidsDT[`Family type by child dependency status` == "Total families",
                 censusConstraint := "nkids_totalFamilies"]

table(kidsDT$`Family type by child dependency status`, kidsDT$censusConstraint)

# > convert to wide
kidsDT <- kidsDT[, count := Value]

kids2013WDT <- reshape(kidsDT[YEAR == 2013,
                              .(AU2013_code,AU2013_label,censusConstraint,count)], 
                          idvar = c("AU2013_code", "AU2013_label"),
                          timevar = "censusConstraint",
                          direction = "wide")
# > calculate n households with 0 kids
# do this again later when we have the number of _households_ to use as the base
kids2013WDT <- kids2013WDT[, nKids_0_families := count.nkids_totalFamilies - count.nKids_1m]
setnames(kids2013WDT, c("count.nkids_totalFamilies", "count.nKids_1m"), 
         c("nkids_totalFamilies", "nKids_1m"))


# nPeople ----
npeoplef <- paste0(p_Params$dataPath, "raw/areaUnits/nPeople/TABLECODE8169_Data_bfad6f1a-c9af-4adb-a141-e13a83e175d0.csv")
message("Loading: ", npeoplef)
npeopleDT <- data.table::fread(npeoplef)
message("N rows loaded: ", dkUtils::tidyNum(nrow(npeopleDT))
        )
setkey(npeopleDT, Area) # forgot to get code attached to this one
npeopleDT <- npeopleDT[, AU2013_label := Area]
setkey(auListDT,AU2013_label) # set to label instead of code - should still work
npeopleDT <- npeopleDT[auListDT]
message("N unique area units (people data): ", uniqueN(npeopleDT$Area))

# > create categories ----
# "value.nPeople_1", "value.nPeople_2", "value.nPeople_3", "value.nPeople_4m",
npeopleDT <- npeopleDT[, censusConstraint := "nPeople_4m"] # default (most complex to code)
npeopleDT <- npeopleDT[`Number of usual residents in household` == "Total households",
                       censusConstraint := "npeople_totalHouseholds"] 
npeopleDT <- npeopleDT[`Number of usual residents in household` %like% "One",
                       censusConstraint := "nPeople_1"] 
npeopleDT <- npeopleDT[`Number of usual residents in household` %like% "Two",
                       censusConstraint := "nPeople_2"] 
npeopleDT <- npeopleDT[`Number of usual residents in household`  %like% "Three",
                       censusConstraint := "nPeople_3"] 
table(npeopleDT$`Number of usual residents in household`, npeopleDT$censusConstraint)

# convert to wide
npeopleDT <- npeopleDT[, count := Value]

npeople2013WDT <- reshape(npeopleDT[Year == 2013, # helpful consistency of var names across NZ stats tables
                              .(AU2013_code,AU2013_label,censusConstraint,count)], 
                          idvar = c("AU2013_code", "AU2013_label"),
                          timevar = "censusConstraint",
                          direction = "wide")

setnames(npeople2013WDT, c("count.nPeople_1", "count.nPeople_2", "count.nPeople_3", "count.nPeople_4m", "count.npeople_totalHouseholds"), 
         c("nPeople_1", "nPeople_2", "nPeople_3", "nPeople_4m", "npeople_totalHouseholds"))

# nRooms ----
nroomsf <- paste0(p_Params$dataPath, "raw/areaUnits/nRooms/TABLECODE8098_Data_62c5ce5c-23cf-44a2-b25e-b287fe9645e7.csv")
message("Loading: ", nroomsf)
nroomsDT <- data.table::fread(nroomsf)
message("N rows loaded: ", dkUtils::tidyNum(nrow(nroomsDT))
        )
nroomsDT <- nroomsDT[, AU2013_code := AREA]
setkey(nroomsDT, AU2013_code)
setkey(auListDT,AU2013_code) # set to back to code
nroomsDT <- nroomsDT[auListDT]
message("N unique area units (rooms data): ", uniqueN(nroomsDT$AU2013_code))

# > create categories ----
# "value.nRooms1_4", "value.nRooms5_6", "value.nRooms7m",
nroomsDT <- nroomsDT[`Number of rooms` != "Not elsewhere included", censusConstraint := "nRooms7m"] # default (most complex to code)
nroomsDT <- nroomsDT[`Number of rooms` == "Total dwellings stated",
                       censusConstraint := "nrooms_statedtotalHouseholds"] 
nroomsDT <- nroomsDT[`Number of rooms` == "Total dwellings, number of rooms",
                       censusConstraint := "nrooms_totalHouseholds"] 
nroomsDT <- nroomsDT[`Number of rooms` %like% "One" |
                       `Number of rooms` %like% "Two" |
                       `Number of rooms` %like% "Three" |
                       `Number of rooms` %like% "Four",
                       censusConstraint := "nRooms1_4"] 
nroomsDT <- nroomsDT[`Number of rooms` %like% "Five" | 
                       `Number of rooms` %like% "Six" ,
                       censusConstraint := "nRooms5_6"] 
table(nroomsDT$`Number of rooms`, nroomsDT$censusConstraint)

# convert to wide
nroomsDT <- nroomsDT[, count := Value]

nrooms2013WDT <- reshape(nroomsDT[Year == 2013, # helpful consistency of var names across NZ stats tables
                              .(AU2013_code,AU2013_label,censusConstraint,count)], 
                          idvar = c("AU2013_code", "AU2013_label"),
                          timevar = "censusConstraint",
                          direction = "wide")

setnames(nrooms2013WDT, c("count.nRooms1_4", "count.nRooms5_6", "count.nRooms7m",
                          "count.nrooms_totalHouseholds", "count.nrooms_statedtotalHouseholds"), 
         c("nRooms1_4", "nRooms5_6", "nRooms7m",
           "nrooms_totalHouseholds", "nrooms_statedtotalHouseholds"))

# combine them into 1 filw ----
setkey(auListDT, AU2013_code, AU2013_label)
setkey(fuel2013WDT, AU2013_code, AU2013_label)
setkey(kids2013WDT, AU2013_code, AU2013_label)
setkey(npeople2013WDT, AU2013_code, AU2013_label)
setkey(nrooms2013WDT, AU2013_code, AU2013_label)

au2013DT <- fuel2013WDT[auListDT]
au2013DT <- kids2013WDT[au2013DT]
au2013DT <- npeople2013WDT[au2013DT]
au2013DT <- nrooms2013WDT[au2013DT]

# recalulate n households without children
au2013DT <- au2013DT[, nKids_0 :=  nrooms_totalHouseholds- nKids_1m]

# Check household totals ----
# These will vary depending on the source table (families vs households vs dwellings etc)
pairsDT <- au2013DT[, .(fuel_totalHouseholds, nrooms_statedtotalHouseholds, nrooms_totalHouseholds,
                        npeople_totalHouseholds,fuel_totalStatedHouseholds, nkids_totalFamilies)]

pairs(pairsDT)

# We focus on households/families/dwellings not individuals as the spatial microsimulation will operate at the household level.

# save data for re-use ----
of <- paste0(p_Params$dataPath, "processed/2013IpfInput.csv")
data.table::fwrite(au2013DT, of)
message("Data saved as: ", of)

